/*
 * Copyright 2008, Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

import java.util.Vector;

public class ExactMineFieldBuilder extends MineFieldBuilder {

    private Data d;

    public static abstract class Data {
	private int w, h;
	private boolean wrapX = false;
	private boolean wrapY = false;
	private boolean[][] mines;
	private boolean[][] uncovered;
	private final MineField mf;
	private final int numMines;

	interface MFB {
	    MineField build(int w, int h, boolean wrapX, boolean wrapY);
	}

	public Data(String str, MFB mfb) {
	    w = 0;
	    h = 0;
	    int i = 0;

	    Vector lines = new Vector();
	    for (int j = 0; j < str.length(); ++j) {
		if (str.charAt(j) == '|') {
		    String line = str.substring(i, j);
		    lines.addElement(line);
		    if (w == 0)
			w = line.length();
		    else if (w != line.length())
			throw new RuntimeException("Lines must have the same length.");

		    i = j + 1;
		    h += 1;
		}
	    }

	    mines = new boolean[w][h];
	    uncovered = new boolean[w][h];
	    int numMines = 0;
	    for (int y = 0; y < h; ++y) {
		String line = (String)lines.elementAt(y);
		for (int x = 0; x < w; ++x) {
		    char c = line.charAt(x);
		    mines[x][y] = c == '1';
		    uncovered[x][y] = c == ' ';
		    if (mines[x][y])
			numMines++;
		}
	    }
	    this.numMines = numMines;

	    this.mf = mfb.build(w, h, wrapX, wrapY);
	}
    }

    public static class Data8 extends Data {
	public Data8(String str, final ParticipantSelector parSel) {
	    super (str, new MFB() {
		    public MineField build(int w, int h,
					   boolean wrapX, boolean wrapY) {
			return new MineField8(w, h, wrapX, wrapY, parSel);
		    }
		});
	}
    }

    public static class Data6 extends Data {
	public Data6(String str, final ParticipantSelector parSel) {
	    super (str, new MFB() {
		    public MineField build(int w, int h,
					   boolean wrapX, boolean wrapY) {
			return new MineField6(w, h, wrapX, wrapY, parSel);
		    }
		});
	}
    }

    public ExactMineFieldBuilder(Data d) {
	super(d.mf);
	this.d = d;
    }

    public void doInitialize(byte idx1) {
	byte idx2 = 0;
	for (int y = 0; y < d.mines[0].length; ++y)
	    for (int x = 0; x < d.mines.length; ++x) {
		Field f = mineField.getField(idx2++);
		f.mine = d.mines[x][y];
		f.uncovered = d.uncovered[x][y];
	    }

	mineField.updateParticipantCounts();
    }

    public int getNumMines() {
	return d.numMines;
    }
}
