/*
 * Copyright 2009 Petr Machata
 *
 * This file is part of MicroSweeper.
 *
 * MicroSweeper is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * MicroSweeper is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with MicroSweeper.  If not, see <http://www.gnu.org/licenses/>.
 */

class NormalField
    extends Field
{
    public NormalField(byte fieldIdx) {
	super(fieldIdx);
    }

    public NormalField(Field copy) {
	super(copy);
    }

    public Field clone() {
	return new NormalField(this);
    }

    public Constraint getConstraint(final MineField mf) {
    	class MyConstraint extends Constraint {
	    MyConstraint(final MineField mf) {
		final byte fieldIdx = NormalField.this.fieldIdx;
		mf.allParticipants(fieldIdx, new FieldEnumerator() {
			public boolean field(Field f) {
			    if (!f.uncovered)
				MyConstraint.this.variables.add(f.fieldIdx);
			    return true;
			}
		    });

		result = mf.getField(fieldIdx).getParCount();
		designator = "x" + fieldIdx;
	    }
	}

    	return new MyConstraint (mf);
    }

    public String render() {
	return parCount == 0 ? ""
	    : Long.toString(parCount);
    }

    public int getColor() {
	final int[] colors = {0,
			      0x000000ff, 0x0000ff00,
			      0x0000ffff, 0x00ff00ff,
			      0x00ffff00};
	if (parCount >= colors.length)
	    return 0x00ff0000;
	else
	    return colors[parCount];
    }

    public byte countParticipantFlags(MineField mineField, byte fieldIdx) {
	return mineField.countParticipantPred(fieldIdx, new FieldEnumerator() {
		public boolean field(Field f) {
		    return f.flagged;
		}
	    });
    }

    public boolean consistent(MineField mineField) {
	return countParticipantFlags(mineField, fieldIdx) <= getParCount();
    }
}
